
#include <avr/io.h>
#include <stdint.h>
#include <avr/interrupt.h>
#include "MyRand.h"

#define COLS_PORT 	PORTB
#define COLS_DDR	DDRB
#define COLS_MASK	0b11111110

#define ROWS_PORT	PORTD
#define ROWS_DDR	DDRD
#define ROWS_MASK	0b00111111

#define DCF_PORT	PORTD
#define DCF_DDR		DDRD
#define DCF_PIN		PIND
#define DCF_MASK	0b01000000

#define BTN_PORT	PORTB
#define BTN_DDR		DDRB
#define BTN_PIN		PINB
#define BTN_MASK	0b00000001

#define REFRESH_INTERVAL 20

static volatile struct
{
	uint8_t bRedraw:1;
	uint8_t bSetHours:1;	//Setting mode Hours
	uint8_t bSetMinutes:1;	//Setting mode Minutes
} g_Flags =
{
	1, 0, 0
};

static volatile struct
{
	uint8_t nHours;
	uint8_t nMinutes;
	uint8_t nSeconds;
} g_Time =
{
	13, 37, 00
};

static volatile uint8_t g_ShortReleasedKeys = 0;
static volatile uint8_t g_LongReleasedKeys = 0;
static uint8_t g_nRow;
static volatile uint8_t g_DisplayBuffer[6] =
{
//  LEFT P RIGHT
//     | | |
//    +++|+++
//    vvvvvvv
	0b00010000,
	0b00000000,
	0b00000000,
	0b00010000,
	0b00000000,
	0b00000000
};



void DrawTime(uint8_t nHours, uint8_t nMinutes, uint8_t bShuffled);
void Shuffle(uint8_t nArray[9]);

ISR(TIMER0_OVF_vect)
{
	ROWS_PORT = ROWS_MASK;
	COLS_PORT = (COLS_PORT & (~COLS_MASK)) | (g_DisplayBuffer[g_nRow] & COLS_MASK);
	ROWS_PORT = (ROWS_PORT & (~ROWS_MASK)) | ((~(1<<g_nRow)) & ROWS_MASK);

	g_nRow++;
	if (g_nRow == 6)
		g_nRow = 0;


  static uint8_t ct0, ct1, KeyState;
  static uint16_t PressureTime = 0;
  uint8_t i;


	i = KeyState ^ ~(BTN_PIN & BTN_MASK);  // key changed ?
	ct0 = ~( ct0 & i );    // reset or count ct0
	ct1 = (ct0 ^ ct1) & i;    // reset or count ct1
	i &= ct0 & ct1;    // count until roll over
	KeyState ^= i;    // then toggle debounced state

	if (KeyState & BTN_MASK)
	{
		if (PressureTime < 65535)
			PressureTime++;
	}else
	{
		if (PressureTime > 500)
		{
			g_LongReleasedKeys |= ~KeyState & i;
			PressureTime = 0;
		}else if (PressureTime > 10)
		{
			g_ShortReleasedKeys |= ~KeyState & i;  // 0->1: key release detect
			PressureTime = 0;
		}
	}


}

ISR(TIMER1_COMPA_vect)
{
	if ( g_Flags.bSetHours || g_Flags.bSetMinutes )
		return;	//not in setting mode

	g_DisplayBuffer[0] ^= (1<<4);
	g_DisplayBuffer[1] ^= (1<<4);
	g_DisplayBuffer[2] ^= (1<<4);
	g_DisplayBuffer[3] ^= (1<<4);

	g_Time.nSeconds++;
	if (g_Time.nSeconds == 60)
	{
		g_Time.nSeconds = 0;

		g_Time.nMinutes++;
		if (g_Time.nMinutes == 60)
		{
			g_Time.nMinutes = 0;

			g_Time.nHours++;
			if (g_Time.nHours == 24)
			{
				g_Time.nHours = 0;
			}
		}
	}

	if ((g_Time.nSeconds % REFRESH_INTERVAL) == 0)
		g_Flags.bRedraw = 1;
}

uint8_t IsReleasedShort()
{
	uint8_t Temp;

  	cli();
  	Temp = g_ShortReleasedKeys;
  	g_ShortReleasedKeys = 0;
  	sei();

  	return Temp;
}

uint8_t IsReleasedLong()
{
	uint8_t Temp;

  	cli();
  	Temp = g_LongReleasedKeys;
  	g_LongReleasedKeys = 0;
  	sei();

  	return Temp;
}



void Prepare(uint8_t nArray[9])
{
	uint8_t i;

	for (i=0; i<9; i++)
	{
		nArray[i] = i;
	}
}

void Shuffle(uint8_t nArray[9])
{
	uint8_t i;

	for (i=0; i<30; i++)
	{
		uint8_t* pElement1 = &nArray[MyRandGet() % 9];
		uint8_t* pElement2 = &nArray[MyRandGet() % 9];

		uint8_t nTmp = *pElement1;
		*pElement1 = *pElement2;
		*pElement2 = nTmp;
	}
}

void DrawTime(uint8_t nHours, uint8_t nMinutes, uint8_t bShuffled)
{
	uint8_t nRed;
	uint8_t nGreen;
	uint8_t aRandomTable[9];
	uint8_t i;

	for (i=0; i<6; i++)
	{
		g_DisplayBuffer[i] &= (1<<4);
	}

	nRed = nHours / 10;
	nGreen = nHours % 10;

	if (nRed != 0)
	{
		Prepare(aRandomTable);
		if (bShuffled) Shuffle(aRandomTable);

		while(nRed--)
		{
			uint8_t nRandomPos = aRandomTable[nRed];

			g_DisplayBuffer[((nRandomPos)/3) * 2 + 1] |= (1<<(nRandomPos%3)) << 1;
		}
	}

	if (nGreen != 0)
	{
		Prepare(aRandomTable);
		if (bShuffled) Shuffle(aRandomTable);

		while(nGreen--)
		{
			uint8_t nRandomPos = aRandomTable[nGreen];

			g_DisplayBuffer[((nRandomPos)/3) * 2] |= (1<<(nRandomPos%3)) << 1;
		}
	}

	nRed = nMinutes / 10;
	nGreen = nMinutes % 10;

	if (nRed != 0)
	{
		Prepare(aRandomTable);
		if (bShuffled) Shuffle(aRandomTable);

		while(nRed--)
		{
			uint8_t nRandomPos = aRandomTable[nRed];

			g_DisplayBuffer[((nRandomPos)/3) * 2 + 1] |= (1<<(nRandomPos%3)) << 5;
		}
	}

	if (nGreen != 0)
	{
		Prepare(aRandomTable);
		if (bShuffled) Shuffle(aRandomTable);

		while(nGreen--)
		{
			uint8_t nRandomPos = aRandomTable[nGreen];

			g_DisplayBuffer[((nRandomPos)/3) * 2] |= (1<<(nRandomPos%3)) << 5;
		}
	}
}

int main()
{
	ROWS_PORT |= ROWS_MASK;		//Highside off
	COLS_DDR |= COLS_MASK;		//Output
	ROWS_DDR |= ROWS_MASK;		//Output
	DCF_DDR &= ~DCF_MASK;		//Input
	BTN_DDR &= ~BTN_MASK;		//Input
	BTN_PORT |= BTN_MASK;		//Pullup on Button

	TCCR0B = (3<<CS00);
	TIMSK |= (1<<TOIE0);

	TCCR1B = (4<<CS10) | (1<<WGM12);
	OCR1A = 31249;
	TIMSK |= (1<<OCIE1A);


	sei();

	while(1)
	{
		if (g_Flags.bRedraw)
		{

			if (g_Flags.bSetHours)
				DrawTime(g_Time.nHours, 0, 0);
			else if (g_Flags.bSetMinutes)
				DrawTime(0, g_Time.nMinutes, 0);
			else
				DrawTime(g_Time.nHours, g_Time.nMinutes, 1);

			cli();
			g_Flags.bRedraw = 0;
			sei();
		}



		uint8_t ShortPress = IsReleasedShort();
		uint8_t LongPress = IsReleasedLong();

		if (g_Flags.bSetHours)
		{
			if (LongPress)
			{
				g_Flags.bSetHours = 0;
				g_Flags.bSetMinutes = 1;
				g_Flags.bRedraw = 1;
			}
			if (ShortPress)
			{
				g_Time.nHours = (g_Time.nHours + 1) % 24;
				g_Flags.bRedraw = 1;
			}



		}else if (g_Flags.bSetMinutes)
		{
			if (LongPress)
			{
				g_Flags.bSetMinutes = 0;
				g_Flags.bRedraw = 1;
			}
			if (ShortPress)
			{
				g_Time.nMinutes = (g_Time.nMinutes + 1) % 60;
				g_Flags.bRedraw = 1;
			}
		}else
		{
			if (LongPress)
			{
				g_Flags.bSetHours = 1;
				g_Flags.bRedraw = 1;
			}
		}



	}

	return 0;
}




